plugins {
    id("com.android.application")
    id("kotlin-android")
    id("kotlin-kapt")
    id("com.google.gms.google-services")
    id("com.google.firebase.crashlytics")
    id("com.google.devtools.ksp")
}

android {
    namespace = "es.verdnatura"
    compileSdk = 35

    defaultConfig {
        applicationId = "es.verdnatura"
        minSdk = 26
        targetSdk = 33 // se deja con target si no Play Protect la bloquea
        versionCode = 361
        versionName = "24.46"
        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
    }


    buildTypes {
        getByName("release") {
            //debuggable = false
            //shrinkResources = true
            //minifyEnabled = true
            proguardFiles(
                getDefaultProguardFile("proguard-android-optimize.txt"), "proguard-rules.pro"
            )
        }
    }
    //package de la app general =   "package_name": "es.verdnatura"
    //package de la app beta    =   "package_name": "es.verdnatura.sfusion"

    applicationVariants.all { variant ->
        variant.outputs.all { output ->
            val flavorName = variant.productFlavors[0].name
            val buildTypeName = variant.buildType.name

            val apkName = when (flavorName) {
                "beta" -> "vn-pickingBeta-${buildTypeName}.apk"
                else -> "vn-picking-${buildTypeName}.apk"
            }
            output.outputFile.renameTo(File(output.outputFile.parentFile, apkName))
            true
        }

    }
    flavorDimensions += "version"
    productFlavors {
        create("beta") {
            applicationIdSuffix = ".sfusion"
        }
        create("general") {}
    }

    buildFeatures {
        viewBinding = true
        //compose = true

        /*kotlinOptions {
            jvmTarget = "1.8"
        }
        buildFeatures {
            compose = true
        }

        composeOptions {
            kotlinCompilerExtensionVersion = "1.4.3"
        }
        */
        dataBinding = true
    }

    compileOptions {
        sourceCompatibility = JavaVersion.VERSION_21
        targetCompatibility = JavaVersion.VERSION_21
    }
    /* composeOptions {
         kotlinCompilerExtensionVersion = "1.5.15"
     }*/
    /* kotlin {
         jvmToolchain(17)
     }
      kotlinOptions {
          jvmTarget = "17"
      }*/

    dependencies {
        implementation(fileTree(mapOf("dir" to "libs", "include" to listOf("*.jar"))))
        implementation(libs.firebase.crashlytics.ktx)
        implementation(libs.firebase.analytics.ktx)
        implementation(libs.play.services.location)
        implementation(libs.navigation.fragment.ktx)
        implementation(libs.androidx.navigation.ui.ktx)
        implementation(libs.androidx.core.ktx)
        implementation(libs.androidx.recyclerview)
        implementation(libs.androidx.room.ktx)
        kapt(libs.androidx.room.compiler)
        implementation(libs.ink)
        implementation(libs.kotlin.stdlib.jdk7)
        implementation(libs.androidx.appcompat)
        implementation(libs.androidx.core.ktx)
        implementation(libs.androidx.constraintlayout)
        implementation(libs.androidx.legacy.support.v4)
        implementation(libs.androidx.lifecycle.extensions)
        implementation(libs.material)
        implementation(libs.retrofit)
        implementation(libs.converter.gson)
        implementation(libs.converter.scalars)
        implementation(libs.lottie)
        implementation(libs.androidx.swiperefreshlayout)
        implementation(libs.androidx.fragment.ktx)
        implementation(libs.androidx.lifecycle.viewmodel.ktx)
        implementation(libs.androidx.lifecycle.livedata.ktx)
        implementation(libs.androidx.legacy.support.v4)
        implementation(libs.koin.core)
        implementation(libs.koin.android)
        implementation(libs.koin.androidx.scope)
        implementation(libs.koin.androidx.viewmodel)
        implementation(libs.glide)
        implementation(libs.okhttp3.integration)
        ksp(libs.glide.ksp)
        implementation(libs.firebase.analytics)
        implementation(libs.android.image.picker)
        implementation(libs.androidx.datastore.preferences)
        implementation(libs.androidx.datastore.core)
        implementation(libs.zxing.android.embedded) { isTransitive = false }
        implementation(libs.core)

        // Compose
        //implementation(libs.androidx.compose.bom)
        /*   implementation("androidx.compose.ui:ui")
           implementation("androidx.compose.material:material")
           implementation("androidx.compose.runtime:runtime")
           implementation("androidx.activity:activity-compose:1.8.2")
           implementation("androidx.compose.ui:ui-graphics")
           implementation("androidx.compose.ui:ui-tooling-preview") */
    }
}

