package es.verdnatura.presentation.common

import android.app.AlertDialog
import android.content.Context
import android.text.InputType
import android.view.View
import es.verdnatura.R
import es.verdnatura.domain.toast
import es.verdnatura.presentation.view.component.CustomDialogList

class PrinterDialogManager(private val context: Context) {
    fun showPrintDialog(
        item: Long, itemName: String, onPrintClick: (Long, String, Int?, Int) -> Unit
    ) {
        val customDialogList = CustomDialogList(context)
        customDialogList.getEditText().setRawInputType(InputType.TYPE_CLASS_NUMBER)
        customDialogList.getEditTextThree().setRawInputType(InputType.TYPE_CLASS_NUMBER)

        customDialogList.setTitle(context.getString(R.string.itemName, item, itemName))
            .setOkButton(context.getString(R.string.printQr)) {
                handlePrintClick(item, customDialogList, onPrintClick, "qr")
                customDialogList.dismiss()
            }.setOkTwoButton(context.getString(R.string.printBarcode)) {
                handlePrintClick(item, customDialogList, onPrintClick, "barcode")
                customDialogList.dismiss()

            }.setKoButton(context.getString(R.string.cancel)) {
                customDialogList.dismiss()
            }.setHintValueThree(context.getString(R.string.labelNumber))
            .setHintValue(context.getString(R.string.optionalPacking)).setTextThree(View.VISIBLE)
            .show()
        customDialogList.getFocusThree()
    }

    private fun handlePrintClick(
        item: Long,
        customDialogList: CustomDialogList,
        onPrintClick: (Long, String, Int?, Int) -> Unit,
        labelType: String
    ) {
        try {
            onPrintClick(
                item,
                labelType,
                if (customDialogList.getValue().isEmpty()) null else customDialogList.getValue()
                    .toInt(),
                if (customDialogList.getValueOptional()
                        .isEmpty()
                ) 1 else customDialogList.getValueOptional().toInt()
            )
            customDialogList.dismiss()
        } catch (ex: Exception) {
            context.getString(R.string.errorInput).toast(context)
        }

    }
}

class LabelDialogHelper(private val context: Context) {
    fun showLabelDialog(
        onItemSelected: (Int) -> Unit,
    ) {
        val builder = AlertDialog.Builder(context)
        builder.setTitle(context.getString(R.string.selectLabeltoPrint))
        val labelCount = Array(10) { (it + 1).toString() }
        builder.setItems(labelCount) { _, which ->
            onItemSelected(which + 1)
        }
        builder.create().show()
    }
}