package es.verdnatura.presentation.view.commom

import android.content.Context
import android.graphics.Color
import android.text.Spannable
import android.text.SpannableString
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.appcompat.widget.SearchView
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.domain.toast

class SearchableAdapter(
    private var listElements: MutableList<NameWithId>,
    private var context: Context,
    private val onItemClick: (NameWithId) -> Unit,
) : RecyclerView.Adapter<SearchableAdapter.NameViewHolder>() {

    private var listElementsFiltered = listElements.toMutableList()
    private var currentQuery: String = ""
    private lateinit var searchView: SearchView

    inner class NameViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        private val nameText: TextView = itemView.findViewById(android.R.id.text1)

        fun bind(nameWithId: NameWithId, query: String) {
            val spannableString = SpannableString(nameWithId.name)
            try {
                if (query.isNotEmpty()) {
                    val startIndex = nameWithId.name.indexOf(query, ignoreCase = true)
                    if (startIndex >= 0) {
                        val endIndex = startIndex + query.length
                        spannableString.setSpan(
                            ForegroundColorSpan(Color.RED),
                            startIndex,
                            endIndex,
                            Spannable.SPAN_EXCLUSIVE_EXCLUSIVE
                        )
                    }
                }
                nameText.text = spannableString
                itemView.setOnClickListener {
                    onItemClick(nameWithId)
                }
            } catch (ex: Exception) {
                ex.message?.toast(context)
            }
        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): NameViewHolder {
        val view = LayoutInflater.from(parent.context)
            .inflate(android.R.layout.simple_list_item_1, parent, false)
        return NameViewHolder(view)
    }

    override fun onBindViewHolder(holder: NameViewHolder, position: Int) {
        val nameWithId = listElementsFiltered[position]
        holder.bind(nameWithId, currentQuery)
    }

    fun filter(query: String) {
        currentQuery = query
        listElementsFiltered = if (query.isEmpty()) {
            listElements.toMutableList()
        } else {
            listElements.filter { it.name.contains(query, ignoreCase = true) }
                .toMutableList()
        }
        notifyDataSetChanged()
    }

    override fun getItemCount(): Int = listElementsFiltered.size

    fun updateList(newList: MutableList<NameWithId>) {
        listElements = newList
        filter("")
    }
}

data class NameWithId(
    val id: Number,
    val name: String
)
