package es.verdnatura.presentation.view.component

import android.app.Dialog
import android.content.Context
import android.view.View
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.textfield.TextInputEditText
import es.verdnatura.R
import es.verdnatura.databinding.ComponentCustomListDialogBinding

class CustomDialogList(context: Context) : Dialog(context, R.style.DialogTheme) {

    private var binding: ComponentCustomListDialogBinding =
        ComponentCustomListDialogBinding.inflate(layoutInflater)

    init {
        setContentView(binding.root)
        getEditText().requestFocus()
    }

    fun getRecyclerView(): RecyclerView {
        return binding.itemRecyclerview
    }

    fun getEditText(): TextInputEditText {
        return binding.customDialogValue
    }

    fun getEditTextTwo(): TextInputEditText {
        return binding.customDialogValueTwo
    }

    fun getEditTextThree(): TextInputEditText {
        return binding.customDialogValueThree
    }

    fun getValue(): String {
        return binding.customDialogValue.text.toString()
    }

    fun getValueOptional(): String {
        return binding.customDialogValueThree.text.toString()
    }

    fun setValue(value: String): CustomDialogList {
        binding.customDialogValue.setText(value)
        binding.textinputlayoutUsername.visibility = View.VISIBLE
        return this
    }

    fun setValueVisibility(modifier: Int) {
        binding.textinputlayoutUsername.visibility = modifier

    }

    fun getValueTwo(): String {
        return binding.customDialogValueTwo.text.toString()
    }

    fun setValueTwo(value: String): CustomDialogList {
        binding.customDialogValueTwo.setText(value)
        binding.textinputlayoutTwo.visibility = View.VISIBLE
        return this
    }

    fun setTitle(title: String): CustomDialogList {
        binding.customDialogTitle.visibility = View.VISIBLE
        binding.customDialogTitle.text = title
        return this
    }

    fun setDescription(title: String): CustomDialogList {
        binding.customDialogDescription.visibility = View.VISIBLE
        binding.customDialogDescription.text = title
        return this
    }

    fun setOkButton(text: String, onButtonClicked: () -> Unit): CustomDialogList {

        binding.customDialogButtonOk.visibility = View.VISIBLE
        binding.customDialogButtonOk.text = text
        binding.customDialogButtonOk.setOnClickListener { onButtonClicked() }
        return this
    }

    fun setOkTwoButton(text: String, onButtonClicked: () -> Unit): CustomDialogList {
        binding.customDialogButtonOkTwo.visibility = View.VISIBLE
        binding.customDialogButtonOkTwo.text = text
        binding.customDialogButtonOkTwo.setOnClickListener { onButtonClicked() }
        return this
    }

    fun setKoButton(text: String, onButtonClicked: () -> Unit): CustomDialogList {
        binding.customDialogButtonKo.visibility = View.VISIBLE
        binding.customDialogButtonKo.text = text
        binding.customDialogButtonKo.setOnClickListener {
            onButtonClicked()
            dismiss()
        }
        return this
    }

    fun setHintValue(text: String): CustomDialogList {
        binding.customDialogValue.hint = text
        return this
    }

    fun setHintValueTwo(text: String): CustomDialogList {
        binding.customDialogValueTwo.hint = text
        return this
    }

    fun setHintValueThree(text: String): CustomDialogList {
        binding.customDialogValueThree.hint = text
        return this
    }

    fun setTextTwoVisibility(visibility: Int): CustomDialogList {
        binding.textinputlayoutTwo.visibility = visibility
        return this
    }

    fun setTextThree(visibility: Int): CustomDialogList {
        binding.textinputlayoutThree.visibility = visibility
        return this
    }

    fun hideDialog(): CustomDialogList {
        binding.customDialogValue.visibility = View.GONE
        return this
    }

    fun showDialog(): CustomDialogList {
        binding.customDialogValue.visibility = View.VISIBLE
        return this
    }

    fun hideDescription(): CustomDialogList {
        binding.customDialogDescription.visibility = View.GONE
        return this
    }

    fun hideTextInput(visibility: Int = View.INVISIBLE): CustomDialogList {
        binding.textinputlayoutUsername.visibility = visibility
        return this
    }

    fun getFocusThree() {
        binding.customDialogValueThree.requestFocus()
    }
}