package es.verdnatura.presentation.view.feature.delivery.fragments

import android.graphics.drawable.Drawable
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.gson.Gson
import es.verdnatura.R
import es.verdnatura.databinding.FragmentRouteBinding
import es.verdnatura.db.DeliveryDatabase
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toDateString
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.base.database
import es.verdnatura.presentation.common.GeneralAdapter
import es.verdnatura.presentation.common.GeneralItem
import es.verdnatura.presentation.common.ItemDescorationCustomized
import es.verdnatura.presentation.common.OnGeneralItemRowClickListener
import es.verdnatura.presentation.common.OnGeneralRowClickListener
import es.verdnatura.presentation.common.OnItemImageRouteRowClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.common.hideKeyboard
import es.verdnatura.presentation.common.itemScanValue
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.delivery.adapters.RouteAdapter
import es.verdnatura.presentation.view.feature.delivery.model.ExpeditionInfoLoadUnload
import es.verdnatura.presentation.view.feature.delivery.model.ExpeditionSalixPosition
import es.verdnatura.presentation.view.feature.delivery.model.RouteInfo
import es.verdnatura.presentation.view.feature.delivery.model.RouteLoaded
import es.verdnatura.presentation.view.feature.delivery.viewmodels.DeliveryViewModel
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.util.Date

class RoutesFragment(
    var title: String = ""
) : BaseFragment<FragmentRouteBinding, DeliveryViewModel>(
    DeliveryViewModel::class
) {

    private var adapter: RouteAdapter? = null
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private lateinit var myListRoute: MutableList<RouteInfo>
    private lateinit var db: DeliveryDatabase
    private lateinit var myListRouteLoaded: List<RouteLoaded>
    private val customDialogList: CustomDialogList by lazy {
        CustomDialogList(requireContext())
    }
    private var listExpeditionScan: ArrayList<GeneralItem> = ArrayList()
    private var listMyRoutes: ArrayList<GeneralItem> = ArrayList()

    private val expeditionScanAdapter: GeneralAdapter? by lazy {
        GeneralAdapter(listExpeditionScan, object : OnGeneralItemRowClickListener {
            override fun onGeneralItemRowClickListener(item: GeneralItem) {

                listExpeditionScan.remove(item)
                expeditionScanAdapter?.notifyDataSetChanged()
            }
        }, onRowClickListener = object : OnGeneralRowClickListener {
            override fun onRowClickListener(item: GeneralItem) {
                getDataForMoveExpeditions(item.code!!)
            }
        }, showDelete = true
        )
    }

    private val recyclerViewConfigured: RecyclerView by lazy {
        customDialogList.getRecyclerView().apply {
            layoutManager =
                LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
            adapter = expeditionScanAdapter
        }
    }

    companion object {
        fun newInstance(title: String) = RoutesFragment(title)
    }

    override fun getLayoutId(): Int = R.layout.fragment_route

    override fun init() {
        super.init()
        setEvents()
        setToolBar()
        binding.scanInput.requestFocus()
        viewModel.getRoutes()

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        pendingOperations()
    }

    private fun pendingOperations() {

        db = database(requireContext().applicationContext)
        if (mobileApplication.hasNetwork) {
            lifecycleScope.launch {
                withContext(Dispatchers.IO) {
                    val pending = db.signedTicketDao().getAll()
                    for (p in pending) {
                        viewModel.saveSign(p.fileName, p.queryMapList)
                    }
                    val pendingExpedition = db.expedicionDao().getAllPendingExpeditions()

                    if (pendingExpedition.isNotEmpty()) {
                        val pendingExpeditionSend = pendingExpedition.map { l ->
                            ExpeditionSalixPosition(
                                expeditionFk = l.id, stateCode = l.code, l.isScanned, l.scanOrder
                            )
                        }

                        viewModel.expeditionStateAddSalix(
                            hashMapOf("expeditions" to pendingExpeditionSend)
                        )
                    }

                }
                db.routesDao().deleteLoaded(toDateString(Date()))
                myListRouteLoaded = db.routesDao().getAllLoaded()

            }
        }

    }

    private fun deleteSignTickets(nameFileSigned: String) {
        lifecycleScope.launch {
            withContext(Dispatchers.IO) {
                db.signedTicketDao().deleteByNameFile(nameFileSigned)
            }

        }
    }

    private fun deleteExpeditionsPending() {
        lifecycleScope.launch {
            withContext(Dispatchers.IO) {
                db.expedicionDao().getDeleteAllState()
            }

        }
    }

    private fun setToolBar() {
        ma.hideBottomNavigation(View.GONE)
        binding.mainToolbar.toolbarSubtitle.setTextColor(requireContext().getColor((R.color.verdnatura_pumpkin_orange)))
        binding.mainToolbar.toolbarTitle.visibility = View.VISIBLE
        binding.mainToolbar.toolbarSubtitle.visibility = View.VISIBLE
        binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
        binding.mainToolbar.toolbarTitle.text = title

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconInfo = ImageView(context)
        val iconRefresh = ImageView(context)
        val iconSearch = ImageView(context)

        iconInfo.setImageResource(R.drawable.ic_info_delivery)
        iconRefresh.setImageResource(R.drawable.ic_info_refresh)
        iconSearch.setImageResource(R.drawable.expedition_find)

        iconInfo.tooltipText = getTooltip(R.drawable.ic_info_delivery)
        iconRefresh.tooltipText = getTooltip(R.drawable.ic_info_refresh)
        iconSearch.tooltipText = getTooltip(R.drawable.expedition_find)

        val iconMove = ImageView(context)
        iconMove.setImageResource(R.drawable.ic_move_expedition)
        iconMove.tooltipText = getString(R.string.moveExpedition)

        //Tarea 8150
        listIcons.add(iconMove)
        listIcons.add(iconInfo)
        listIcons.add(iconRefresh)
        listIcons.add(iconSearch)

        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {

                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconRefresh.drawable -> {
                            viewModel.getRoutes()
                        }

                        iconInfo.drawable -> {
                            ma.onPasillerosItemClickListener(
                                PasillerosItemVO(title = R.string.titleInfo),
                                getString(R.string.titleInfo)
                            )
                        }

                        iconSearch.drawable -> {
                            ma.onPasillerosItemClickListener(
                                PasillerosItemVO(title = R.string.titleLog),
                                getString(R.string.titleLog)
                            )
                        }

                        iconMove.drawable -> {
                            showExpeditionScan()
                        }
                    }
                }
            })

        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
        binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
    }

    private fun showExpeditionScan() {
        listExpeditionScan.clear()
        listMyRoutes.clear()
        expeditionScanAdapter!!.updateItems(listExpeditionScan)
        expeditionScanAdapter!!.setShowDelete(true)
        customDialogList.setTitle(getString(R.string.scanExpeditionForMove))
            .setOkButton(getString(R.string.moveToRoute)) {
                ma.hideKeyboard(customDialogList.getEditText())
                customDialogList.dismiss()
                showRoutes()

            }.setKoButton(getString(R.string.cancel)) {
                customDialogList.dismiss()
                requireContext().hideKeyboard(binding.scanInput)
            }.setValue("").showDialog().show()

        customDialogList.getEditText().requestFocus()
        ma.hideKeyboard(customDialogList.getEditText())
        customDialogList.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (customDialogList.getValue().isNotEmpty()) {
                    val expeditionScan = itemScanValue(
                        customDialogList.getValue(), arrayOf("expeditionPallet"), "id"
                    ).toString()
                    listExpeditionScan.add(
                        GeneralItem(
                            expeditionScan, expeditionScan
                        )
                    )
                    expeditionScanAdapter?.notifyItemChanged(0)
                }
                customDialogList.setValue("")
                ma.hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        recyclerViewConfigured
        requireContext().hideKeyboard(customDialogList.getEditText())
    }

    private fun showRoutes() {
        listMyRoutes.clear()
        expeditionScanAdapter!!.updateItems(
            ArrayList(myListRoute.map { routeInfo ->
                GeneralItem(
                    code = routeInfo.id.toString(), text = routeInfo.id.toString()
                )
            })
        )
        expeditionScanAdapter!!.setShowDelete(false)
        customDialogList.setTitle(getString(R.string.selectRoute))
            .setKoButton(getString(R.string.cancel)) {
                customDialogList.dismiss()
                requireContext().hideKeyboard(binding.scanInput)
            }.setValue("").hideDialog().show()
        requireContext().hideKeyboard(customDialogList.getEditText())
        recyclerViewConfigured

    }

    private fun getDataForMoveExpeditions(routeId: String) {
        val routeItem = myListRoute.find { it.id == routeId.toLong() }
        for (expedition in listExpeditionScan) {
            viewModel.getDataFromExpedition(
                routeId = routeId.toLong(),
                expeditionId = expedition.code!!.toLong(),
                landed = routeItem!!.created.toString()
            )
        }
        customDialogList.dismiss()

    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()

        }


        binding.scanInput.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                filteredList(s.toString().lowercase())
            }

            override fun afterTextChanged(s: Editable?) {
            }
        })

    }

    private fun filteredList(searchText: String) {
        if (::myListRoute.isInitialized) {
            val filteredList = myListRoute.filter { route ->
                route.toString().lowercase().contains(searchText)
            }.toMutableList()
            adapter!!.updateList(filteredList)
        }

    }

    override fun observeViewModel() {
        with(viewModel) {
            loadRouteList.observe(viewLifecycleOwner) { event ->
                event.getContentIfNotHandled().notNull {
                    createListRoutes(it.list)
                }
            }


            routeInfoList.observe(viewLifecycleOwner) {
                if (it.list.isNotEmpty()) {
                    binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
                    binding.mainToolbar.switchButton.visibility = View.VISIBLE
                } else {
                    binding.scanInput.setText("")
                    binding.mainToolbar.toolbarIcons.visibility = View.GONE
                    binding.mainToolbar.switchButton.visibility = View.GONE
                }
            }


            responseSign.observe(viewLifecycleOwner) {

                if (it.isError) {
                    binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
                    it.message.toast(context)

                } else {
                    deleteSignTickets(it.fileToSign)
                }
            }

            responseLoadList.observe(viewLifecycleOwner) {

                ma.onApplicationEventAnimation(View.INVISIBLE)
                createListRoutes(mutableListOf(), getSaved = true)
            }

            responseStateAdd.observe(viewLifecycleOwner) {

                if (it) {
                    binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
                    deleteExpeditionsPending()
                }
            }

        }

    }

    private fun createListRoutes(list: MutableList<RouteInfo>, getSaved: Boolean = false) {

        lifecycleScope.launch {
            withContext(Dispatchers.IO) {

                myListRoute = if (getSaved) {
                    db.routesDao().getAll() as MutableList<RouteInfo>
                } else {
                    db.routesDao().delete()
                    db.routesDao().insert(list)
                    list
                }

            }

            // myListRoute = list

            adapter = RouteAdapter(
                myListRoute, object : OnItemImageRouteRowClickListener {

                    override fun onItemImageRouteRowClickListener(item: RouteInfo, action: String) {
                        when (action) {
                            "loadVan" -> {
                                binding.scanInput.isEnabled = true
                                ma.onPasillerosItemClickListener(
                                    PasillerosItemVO(title = R.string.titleDeliverySummary),
                                    item.id.toString()
                                )
                            }

                            "showTickets" -> {
                                ma.onPasillerosItemClickListener(
                                    PasillerosItemVO(title = R.string.tickets), Gson().toJson(item)
                                )
                            }
                        }

                    }

                }, if (::myListRouteLoaded.isInitialized) {
                    myListRouteLoaded
                } else {
                    listOf()
                }
            )

            binding.routeRecyclerview.addItemDecoration(
                ItemDescorationCustomized(
                    resources.getDimensionPixelSize(
                        R.dimen.vertical_spacing
                    )
                )
            )
            binding.routeRecyclerview.adapter = adapter
            binding.routeRecyclerview.layoutManager =
                LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
            // adapter!!.notifyDataSetChanged()
            if (!binding.scanInput.text.isNullOrBlank()) {
                filteredList(binding.scanInput.text.toString())
            }

        }
    }

    private fun setListPosition(list: MutableList<ExpeditionInfoLoadUnload>) {
        for (i in list.indices) {
            binding.routeRecyclerview.scrollToPosition(i)
            break
        }
    }

}

