package es.verdnatura.presentation.view.feature.delivery.fragments

import android.app.DatePickerDialog
import android.graphics.drawable.Drawable
import android.view.View
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentSalaryComplementBinding
import es.verdnatura.domain.toDateString
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnSalarySupplementClickListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.view.commom.NameWithId
import es.verdnatura.presentation.view.commom.SearchableAdapter
import es.verdnatura.presentation.view.feature.delivery.adapters.SalarySupplementAdapter
import es.verdnatura.presentation.view.feature.delivery.model.RouteActionList
import es.verdnatura.presentation.view.feature.delivery.model.RouteComplement
import es.verdnatura.presentation.view.feature.delivery.model.RouteComplementAdd
import es.verdnatura.presentation.view.feature.delivery.viewmodels.DeliveryViewModel
import java.time.LocalDate
import java.time.format.DateTimeFormatter
import java.util.Calendar
import java.util.Date

class SalaryComplementFragment(
    var entryPoint: String = ""

) : BaseFragment<FragmentSalaryComplementBinding, DeliveryViewModel>(
    DeliveryViewModel::class
) {

    private val adapterComplement: SalarySupplementAdapter by lazy {
        SalarySupplementAdapter(emptyList(), object : OnSalarySupplementClickListener {
            override fun onSalarySupplementClickListener(item: RouteComplement) {
                viewModel.routeComplementDelete(
                    item.id,
                    binding.date.text.toString(),
                    mobileApplication.userId!!
                )
            }
        })
    }

    private val layoutManager: LinearLayoutManager by lazy {
        LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    private lateinit var routeActionComplementsList: RouteActionList

    companion object {
        fun newInstance(entryPoint: String) = SalaryComplementFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_salary_complement

    private fun showDatePicker() {
        val currentDate = Calendar.getInstance()
        val year = currentDate[Calendar.YEAR]
        val month = currentDate[Calendar.MONTH]
        val day = currentDate[Calendar.DAY_OF_MONTH]

        DatePickerDialog(
            requireContext(), { _, selectedYear, selectedMonth, selectedDay ->
                binding.date.text =
                    getString(R.string.dateDay, selectedDay, selectedMonth + 1, selectedYear)
                viewModel.getRouteComplements(
                    binding.date.text.toString(),
                    mobileApplication.userId!!
                )
            }, year, month, day
        ).apply {
            setTitle(getString(R.string.selectDate))
            show()
        }
    }

    override fun init() {
        viewModel.getRouteAction()
        binding.date.text = toDateString(Date())
        viewModel.getRouteComplements(binding.date.text.toString(), mobileApplication.userId!!)
        ma.hideBottomNavigation(View.GONE)
        setToolBar()
        setEvents()
        super.init()
    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarTitle.text = getString(R.string.supplementDelivery)
        binding.mainToolbar.toolbarIcons.visibility = View.VISIBLE
        val iconAdd = ImageView(context).apply {
            setImageResource(R.drawable.ic_add_black_24dp)
            tooltipText = getTooltip(R.drawable.ic_add_black_24dp)
        }

        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listOf(iconAdd), object : OnOptionsSelectedListener {

                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconAdd.drawable -> {
                            binding.salarySearchableRecyclerView.visibility = View.VISIBLE
                        }

                    }

                }

            })

        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }
        binding.date.setOnClickListener {
            showDatePicker()
        }
    }

    private fun setSearchable(listElements: MutableList<NameWithId>) {

        val searchableRecyclerView = binding.salarySearchableRecyclerView
        val adapter =
            SearchableAdapter(
                listElements = listElements,
                context = requireContext()
            ) { nameWithId ->
                val foundItem = routeActionComplementsList.list.find { item ->
                    item.id == nameWithId.id
                }
                foundItem?.apply {
                    val dateFormatted = LocalDate.parse(
                        binding.date.text.toString(), DateTimeFormatter.ofPattern("dd-MM-yyyy")
                    ).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"))
                    viewModel.addRouteComplements(
                        RouteComplementAdd(
                            workerFk = mobileApplication.userId!!,
                            routeActionFk = foundItem.id,
                            price = foundItem.price,
                            dated = dateFormatted,
                        )
                    )

                }
                searchableRecyclerView.visibility = View.GONE
            }
        searchableRecyclerView.setAdapter(adapter, listElements)

    }

    override fun observeViewModel() {
        with(viewModel) {
            routeActionList.observe(viewLifecycleOwner) { list ->
                routeActionComplementsList = list
                setSearchable(list.list.map { item ->
                    NameWithId(
                        id = item.id,
                        name = item.name
                    )
                }.toMutableList())
            }
            routeComplementList.observe(viewLifecycleOwner) {
                adapterComplement.updateData(it.list)
                binding.salaryComplementRecyclerview.adapter = adapterComplement
                binding.salaryComplementRecyclerview.layoutManager = layoutManager
            }
        }
    }
}
