package es.verdnatura.presentation.view.feature.workermistake.fragment

import android.content.Context
import android.os.Bundle
import android.view.View
import androidx.core.widget.addTextChangedListener
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentWorkermistakeBinding
import es.verdnatura.domain.ConstAndValues.DEPARTMENTMISTAKE
import es.verdnatura.domain.ConstAndValues.DEPARTMENTMISTAKEID
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnBarcodeRowClickListener
import es.verdnatura.presentation.common.OnMistakeWorkerClickListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.commom.NameWithId
import es.verdnatura.presentation.view.commom.SearchableAdapter
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.inventario.fragment.InventaryViewModel
import es.verdnatura.presentation.view.feature.workermistake.adapter.WorkermistakeAdapter
import es.verdnatura.presentation.view.feature.workermistake.model.MistakeType
import es.verdnatura.presentation.view.feature.workermistake.model.WorkerFromMistake
import kotlinx.coroutines.runBlocking

class WorkermistakeFragment(
    var entryPoint: String = ""

) : BaseFragment<FragmentWorkermistakeBinding, InventaryViewModel>(
    InventaryViewModel::class
) {
    private var adapterworker: WorkermistakeAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private var listWorker: MutableList<WorkerFromMistake> = ArrayList()
    private var listWorkerAuxiliary: MutableList<WorkerFromMistake> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private var layoutManager: LinearLayoutManager? = null
    private lateinit var customDialogList: CustomDialogList
    private var listMistakes: ArrayList<BarcodeVO> = ArrayList()
    private var listMistakesAdapter: BarcodeAdapter? = null
    private var workerMistakeName: WorkerFromMistake? = null

    companion object {
        fun newInstance(entryPoint: String) = WorkermistakeFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_workermistake

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        viewModel.departmentGetHasMistake()
        super.onCreate(savedInstanceState)
    }

    override fun init() {

        runBlocking {
            binding.filterDepartment.text =
                mobileApplication.dataStoreApp.readDataStoreKey<String>("DEPARTMENTMISTAKE")
            val departmentId =
                mobileApplication.dataStoreApp.readDataStoreKey<Int>("DEPARTMENTMISTAKEID")
            if (departmentId != 0) {
                viewModel.workerGetFromHasMistake(
                    departmentId
                )
            }
        }

        customDialogList = CustomDialogList(requireContext())
        binding.filterDepartment.setOnClickListener {
            binding.searchableRecyclerView.visibility = View.VISIBLE
            binding.searchableRecyclerView.setSearchHint(getString(R.string.departamentSearch))
        }
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        ma.hideBottomNavigation(View.GONE)
        setToolBar()
        setEvents()
        super.init()
    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarTitle.text = entryPoint
    }

    private fun setSearchable(listNames: MutableList<NameWithId>) {

        val adapter =
            SearchableAdapter(
                listElements = listNames,
                context = requireContext()
            ) { elementSelected ->

                runBlocking {
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        DEPARTMENTMISTAKE,
                        elementSelected.name
                    )
                    mobileApplication.dataStoreApp.editDataStoreKey(
                        DEPARTMENTMISTAKEID,
                        elementSelected.id
                    )
                    viewModel.workerGetFromHasMistake(
                        elementSelected.id
                    )
                }
                binding.filterDepartment.text = elementSelected.name
                binding.searchableRecyclerView.visibility = View.GONE

            }

        binding.searchableRecyclerView.setAdapter(adapter, listNames)

    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }
        if (binding.filterDepartment.text.isNullOrEmpty()) {
            binding.filterItemFk.isEnabled = false
        }
        binding.filterItemFk.addTextChangedListener { filter() }

    }

    private fun filter() {

        if (listWorker.isNotEmpty()) {
            listWorker.clear()
            listWorker.addAll(listWorkerAuxiliary.filter {
                it.firstName.uppercase().contains(
                    binding.filterItemFk.text.toString().uppercase()
                ) || it.lastName.uppercase()
                    .contains(binding.filterItemFk.text.toString().uppercase())
            } as MutableList<WorkerFromMistake>)
            adapterworker!!.notifyDataSetChanged()
        }

    }

    override fun observeViewModel() {
        with(viewModel) {
            mistakeDepartmentList.observe(viewLifecycleOwner) {
                setSearchable(it.list.map { item ->
                    NameWithId(
                        id = item.id,
                        name = item.name
                    )
                } as MutableList<NameWithId>)
            }
            workerFromMistakeList.observe(viewLifecycleOwner) {
                createWorkerList(it.list)
                adapterworker =
                    WorkermistakeAdapter(listWorker, object : OnMistakeWorkerClickListener {
                        override fun onMistakeWorkerClickListener(item: WorkerFromMistake) {
                            viewModel.workerMistakeTypeGet(
                            )
                            workerMistakeName =
                                WorkerFromMistake(item.id, item.firstName, item.lastName)
                        }
                    })
                binding.workermistakeRecyclerview.adapter = adapterworker
                layoutManager =
                    LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                binding.workermistakeRecyclerview.layoutManager = layoutManager
            }

            mistakeWorkerList.observe(viewLifecycleOwner) {
                showMistakeList(it.list)
            }
        }
    }

    private fun showMistakeList(list: List<MistakeType>) {
        listMistakes = ArrayList()
        list.forEach {
            listMistakes.add(BarcodeVO(code = it.description))
        }
        customDialogList.setTitle("${workerMistakeName!!.firstName} \n\n ${getString(R.string.errorCause)}")
        customDialogList.hideDescription()
        customDialogList.hideTextInput(View.GONE)
            .setKoButton(
                getString(
                    R.string.cancel
                )
            ) {
                customDialogList.dismiss()
            }.hideDialog().show()

        listMistakesAdapter =
            BarcodeAdapter(listMistakes, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    list.forEach {
                        if (it.description == item.code) {
                            viewModel.workerMistakeAdd(
                                vUserFk = workerMistakeName!!.id!!,
                                vTypeFk = it.code
                            )
                            customDialogList.dismiss()
                        }
                    }

                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = listMistakesAdapter
        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    private fun createWorkerList(list: List<WorkerFromMistake>) {
        try {
            listWorker.apply {
                clear()
                addAll(list)
            }
            listWorkerAuxiliary.apply {
                clear()
                addAll(listWorker)
            }
        } catch (e: Exception) {
            e.message?.toast(requireContext())
        }
    }

}

